<?php

/**
 * Amibios class.
 *
 * @category   apps
 * @package    amibios
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2018 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/amibios/
 */

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\amibios;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('amibios');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\Engine as Engine;
use \clearos\apps\base\File as File;
use \clearos\apps\base\Folder as Folder;
use \clearos\apps\base\Lock as Lock;
use \clearos\apps\base\Shell as Shell;

clearos_load_library('base/Engine');
clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Lock');
clearos_load_library('base/Shell');

// Exceptions
//-----------

use \Exception as Exception;
use \clearos\apps\base\Engine_Exception as Engine_Exception;
use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/Engine_Exception');
clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Amibios class.
 *
 * @category   apps
 * @package    amibios
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2018 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/amibios/
 */

class Amibios_Library extends Engine
{
    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////

    const PATH_FIRMWARE = '/var/clearos/amibios/firmware';
    const PATH_UPLOADS = '/var/clearos/amibios/uploads';
    const PATH_EXTRACT = '/var/clearos/amibios/extract';

    const FILE_FIRMWARE_INFO_LOG = 'amibios_firmware_info.log';
    const FILE_FIRMWARE_INSTALL_LOG = 'amibios_firmware_install.log';
    const FILE_FIRMWARE_BACKUP_LOG = 'amibios_firmware_backup.log';
    const FILE_AFU = 'afulnx_64';

    const COMMAND_AFU = '/var/clearos/amibios/bin/afulnx_64';
    const COMMAND_DMIDECODE = '/sbin/dmidecode';
    const COMMAND_UNZIP = '/bin/unzip';

    const CONSTANT_BIOS = 'American Megatrends';

    const LOCK_INSTALL = 'amibios_lock';
    const LOCK_BACKUP = 'amibios_lock';
    const LOCK_SCAN = 'amibios_lock';
    const LOCK_AMIBIOS = 'amibios_lock';
    const LOCK_AMIBIOS_RUN = '/run/app-amibios.lock';

    const STATUS_UNAVAILABLE = 'unavailable';
    const STATUS_READY = 'ready';
    const STATUS_BUSY = 'busy';

    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Amibios constructor.
     */

    public function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);

        // Throw an error if lock is busy
        $lock = new Lock(self::LOCK_AMIBIOS);

        $max_tries = 0;

        while ($lock->is_locked()) {
            sleep(1);
            $max_tries++;
            
            if ($max_tries > 60)
                throw new Engine_Exception(lang('amibios_busy_try_again_later'));
        }

        // check whatever restart is required
        if ($this->get_run_lock()) {
            throw new Engine_Exception(lang('amibios_restart_required'));
        }
    }

    /**
     * Backup installed firmware.
     *
     * @return array information about backup
     * @throws Engine_Exception
     */

    public function backup_installed_firmware()
    {
        clearos_profile(__METHOD__, __LINE__);

        // Grab the current firmware info
        //-------------------------------

        $current = $this->get_installed_firmware();

        // Only one request allowed at a time - use locking
        //-------------------------------------------------

        $max_tries = 0;
        $lock = new Lock(self::LOCK_BACKUP);

        while ($lock->get_lock() !== TRUE) {
            sleep(1);
            $max_tries++;
            if ($max_tries > 60)
                throw new Engine_Exception(lang('amibios_unable_to_access_amibios_tool'));
        }

        try {

            $date_stamp = date('Y-m-d-his');
            $backup_basename = $current['firmware_id'] . '-' . $date_stamp . '.rom';
            $backup_filename = self::PATH_FIRMWARE . '/' . $backup_basename;

            $file = new File(CLEAROS_TEMP_DIR . '/' . self::FILE_FIRMWARE_BACKUP_LOG);

            if ($file->exists())
                $file->delete();

            $options['validate_exit_code'] = FALSE;
            $options['log'] = self::FILE_FIRMWARE_BACKUP_LOG;

            $shell = new Shell();
            $shell->execute(self::COMMAND_AFU, $backup_filename . ' /O', TRUE, $options);
        } catch (Exception $e) {
            $lock->unlock();
            throw new Engine_Exception(clearos_exception_message($e));
        }

        $lock->unlock();

        $data['filename'] = $backup_basename;

        return $data;
    }

    /**
     * Checks availabilty of AMIBIOS.
     *
     * @return boolean TRUE if available
     */

    public function get_availability()
    {
        clearos_profile(__METHOD__, __LINE__);

        $shell = new Shell();
        $shell->execute(self::COMMAND_DMIDECODE, '', TRUE);

        $lines = $shell->get_output();
        $in_bios_info = FALSE;

        foreach ($lines as $line) {
            if (preg_match('/^BIOS Information/i', $line)) {
                $in_bios_info = TRUE;
            } else if ($in_bios_info) {
                if (preg_match("/\s*Vendor:\s" . self::CONSTANT_BIOS . "/", $line))
                    return TRUE;

                $in_bios_info = FALSE;
            }
        }

        return FALSE;
    }

    /**
     * Returns system's firmware information.
     *
     * @param boolean $use_cache use cache if available
     *
     * @return array system firmware information
     * @throws Engine_Exception
     */

    public function get_installed_firmware($use_cache = TRUE)
    {
        clearos_profile(__METHOD__, __LINE__);

        // Only one request allowed at a time - use locking
        //-------------------------------------------------

        $status = $this->get_status();

        if ($status['code'] != self::STATUS_READY)
            throw new Engine_Exception($status['message']);

        $max_tries = 0;
        $lock = new Lock(self::LOCK_SCAN);

        while ($lock->get_lock() !== TRUE) {
            sleep(1);
            $max_tries++;
            if ($max_tries > 60)
                throw new Engine_Exception(lang('amibios_unable_to_access_amibios_tool'));
        }

        try {

            // If no cache, or no state file exists, run the utility
            //------------------------------------------------------

            $file = new File(CLEAROS_TEMP_DIR . '/' . self::FILE_FIRMWARE_INFO_LOG);

            if (!$use_cache || !$file->exists()) {

                // Delete old state file
                //----------------------

                if ($file->exists())
                    $file->delete();

                // Get firmware ID info
                //---------------

                $options['validate_exit_code'] = FALSE;
                $options['log'] = self::FILE_FIRMWARE_INFO_LOG;

                $shell = new Shell();
                $shell->execute(self::COMMAND_AFU, '/S', TRUE, $options);
            }

            // Parse results
            //--------------

            $lines = $file->get_contents_as_array();

            $result = [];

            foreach ($lines as $line) {
                $matches = [];
                if (preg_match('/\s*-\s*System ROM ID\s*=\s*(.*)/', $line, $matches))
                    $result['firmware_id'] = $matches[1];
                else if (preg_match('/\s*-\s*System ROM GUID\s*=\s*(.*)/', $line, $matches))
                    $result['firmware_guid'] = $matches[1];
                else if (preg_match('/\s*-\s*System ROM Secure Flash\s*=\s*(.*)/', $line, $matches))
                    $result['secure_flash'] = $matches[1];
            }
        } catch (Exception $e) {
            $lock->unlock();
            throw new Engine_Exception(clearos_exception_message($e));
        }

        $lock->unlock();

        return $result;
    }

    /**
     * Returns Amibios status.
     *
     * Installing can take a minute or so, provide a way to show a "busy" status
     *
     * @return array status information
     * @throws Engine_Exception
     */

    public function get_status()
    {
        clearos_profile(__METHOD__, __LINE__);

        if (!$this->get_availability()) {
            $status['code'] = self::STATUS_UNAVAILABLE;
            $status['message'] = lang('amibios_amibios_not_available');
        } else {
            $scan_lock = new Lock(self::LOCK_SCAN);
            $backup_lock = new Lock(self::LOCK_BACKUP);
            $install_lock = new Lock(self::LOCK_INSTALL);

            if ($scan_lock->is_locked()) {
                $status['code'] = self::STATUS_BUSY;
                $status['message'] = lang('amibios_firmware_scan');
            } else if ($backup_lock->is_locked()) {
                $status['code'] = self::STATUS_BUSY;
                $status['message'] = lang('amibios_firmware_backup');
            } else if ($install_lock->is_locked()) {
                $status['code'] = self::STATUS_BUSY;
                $status['message'] = lang('amibios_firmware_installing');
            } else {
                $status['code'] = self::STATUS_READY;
                $status['message'] = '';
            }
        }

        return $status;
    }

    /**
     * Checks availabilty of firmware utilility.
     *
     * @return boolean TRUE if available
     */

    public function get_utility_availability()
    {
        clearos_profile(__METHOD__, __LINE__);

        $file = new File(self::COMMAND_AFU);

        if ($file->exists())
            return TRUE;
        else
            return FALSE;
    }

    /**
     * Create LOCK file for update.
     *
     * @return boolean TRUE if success
     * @return boolean FALSE if file already exists and still need to restart
     */
    public function set_run_lock()
    {
        $file = new File(self::LOCK_AMIBIOS_RUN);

        if ($file->exists())
            return FALSE;

        $file->create('root', 'root');
        return TRUE;
    }

    /**
     * Check whatever run LOCK exists.
     *
     * @return boolean TRUE if success
     * @return boolean FALSE if file already exists and still need to restart
     */
    public function get_run_lock()
    {
        $file = new File(self::LOCK_AMIBIOS_RUN);

        if ($file->exists())
            return TRUE;

        return FALSE;
    }
}
