<?php

/**
 * Amibios firmware files class.
 *
 * @category   apps
 * @package    amibios
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2018 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/amibios/
 */

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\amibios;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('amibios');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\amibios\Amibios_Library as Amibios_Library;
use \clearos\apps\base\Engine as Engine;
use \clearos\apps\base\File as File;
use \clearos\apps\base\Folder as Folder;
use \clearos\apps\base\Lock as Lock;
use \clearos\apps\base\Shell as Shell;

clearos_load_library('amibios/Amibios_Library');
clearos_load_library('base/Engine');
clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Lock');
clearos_load_library('base/Shell');

// Exceptions
//-----------

use \Exception as Exception;
use \clearos\apps\base\Engine_Exception as Engine_Exception;
use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/Engine_Exception');
clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Amibios firmware files class.
 *
 * @category   apps
 * @package    amibios
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2018 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/amibios/
 */

class Firmware_Files_Library extends Engine
{
    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Amibios firmware files constructor.
     */

    public function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);

        // Throw an error if lock is busy
        $lock = new Lock(Amibios_Library::LOCK_AMIBIOS);

        $max_tries = 0;
        
        while ($lock->is_locked()) {
            sleep(1);
            $max_tries++;

            if ($max_tries > 60)
                throw new Engine_Exception(lang('amibios_busy_try_again_later'));
        }

        // check whatever restart is required
        if (Amibios_Library::get_run_lock()) {
            throw new Engine_Exception(lang('amibios_restart_required'));
        }
    }

    /**
     * Deletes firmware from store.
     *
     * @param string $firmware firmware ID
     *
     * @return void
     * @throws Engine_Exception
     */

    public function delete($firmware)
    {
        clearos_profile(__METHOD__, __LINE__);

        Validation_Exception::is_valid($this->validate_firmware($firmware));

        $file = new File(Amibios_Library::PATH_FIRMWARE . '/' . $firmware, TRUE);

        if (!$file->exists())
            throw new Validation_Exception(lang('amibios_firmware_file_does_not_exist'));

        $file->delete();
    }

    /**
     * Returns firmware info.
     *
     * @param string $firmware firmware ID
     *
     * @return array firmware information
     * @throws Engine_Exception
     */

    public function get($firmware)
    {
        clearos_profile(__METHOD__, __LINE__);

        Validation_Exception::is_valid($this->validate_firmware($firmware));

        $all = $this->get_list();

        return $all[$firmware];
    }

    /**
     * Returns available firmware.
     *
     * @return array available firmware
     * @throws Engine_Exception
     */

    public function get_list()
    {
        clearos_profile(__METHOD__, __LINE__);

        $folder = new Folder(Amibios_Library::PATH_FIRMWARE, TRUE);

        $raw_listing = $folder->get_listing();

        $listing = [];

        foreach ($raw_listing as $item) {
            if (preg_match('/\.(rom|ROM)$/', $item)) {
                $filename = Amibios_Library::PATH_FIRMWARE . '/' . $item;
                $file = new File($filename, TRUE);

                $options['validate_exit_code'] = FALSE;
                $shell = new Shell();
                $shell->execute(Amibios_Library::COMMAND_AFU, $filename . ' /U', TRUE, $options);

                // Parse results
                //--------------

                $lines = $shell->get_output();

                foreach ($lines as $line) {
                    $matches = [];
                    if (preg_match('/\s*ROM ID\s*=\s*(.*)/', $line, $matches))
                        $listing[$item]['firmware_id'] = $matches[1];
                    else if (preg_match('/\s*ROM GUID\s*=\s*(.*)/', $line, $matches))
                        $listing[$item]['firmware_guid'] = $matches[1];
                }

                $listing[$item]['size'] = $file->get_size();
            }
        }

        return $listing;
    }

    /**
     * Imports firmware from local store.
     *
     * @param string $zip_filename zip filename
     *
     * @return void
     * @throws Engine_Exception
     */

    public function import_zip($zip_filename)
    {
        clearos_profile(__METHOD__, __LINE__);

        // Validate
        //---------
        
        if (empty($zip_filename))
            throw new Validation_Exception(lang('amibios_import_file_missing'));

        $zip_file = new File($zip_filename, TRUE);
        $zip_file->chmod('0755');
        $zip_size = $zip_file->get_size();

        // Create extract folder
        //----------------------

        $extract_folder = new Folder(Amibios_Library::PATH_EXTRACT);

        if ($extract_folder->exists())
            $extract_folder->delete(TRUE);

        $extract_folder->create('webconfig', 'webconfig', '0755');

        // Handle unzip
        //-------------

        $shell = new Shell();
        $shell->execute(Amibios_Library::COMMAND_UNZIP, '-o -j -d ' . Amibios_Library::PATH_EXTRACT . ' ' . $zip_filename);

        // Copy utility binary
        //--------------------

        $bin_target = new File(Amibios_Library::COMMAND_AFU);
        $bin_source = new File(Amibios_Library::PATH_EXTRACT . '/' . Amibios_Library::FILE_AFU);

        if (!$bin_source->exists())
            throw new Validation_Exception(lang('amibios_no_firmware_tool'));

        $bin_source->chmod('0755');
        $bin_source->move_to(Amibios_Library::COMMAND_AFU);

        // Copy firmware
        //--------------

        $firmware_folder = new Folder(Amibios_Library::PATH_FIRMWARE);
        $zipped_files = $extract_folder->get_listing();

        foreach ($zipped_files as $source_filename) {
            if (preg_match('/\.rom$/', $source_filename)) {
                $source_file = new File(Amibios_Library::PATH_EXTRACT . '/' . $source_filename);
                $source_file->copy_to(Amibios_Library::PATH_FIRMWARE . '/' . $source_filename);
            }
        }

        // Cleanup extract folder
        //-----------------------

        if ($extract_folder->exists())
            $extract_folder->delete(TRUE);

        // Remove cache files
        //-------------------

        $file = new File(CLEAROS_TEMP_DIR . '/' . Amibios_Library::FILE_FIRMWARE_INFO_LOG);

        if ($file->exists())
            $file->delete();
    }

    /**
     * Installs firmware.
     *
     * @param string $firmware firmware ID
     *
     * @return array information about installation
     * @throws Engine_Exception
     */

    public function install($firmware)
    {
        clearos_profile(__METHOD__, __LINE__);

        // Validation
        //-----------

        Validation_Exception::is_valid($this->validate_firmware($firmware));

        // Only one request allowed at a time - use locking
        //-------------------------------------------------

        $max_tries = 0;
        $lock = new Lock(Amibios_Library::LOCK_INSTALL);

        while ($lock->get_lock() !== TRUE) {
            sleep(1);
            $max_tries++;
            if ($max_tries > 60)
                throw new Engine_Exception(lang('amibios_unable_to_access_amibios_tool'));
        }

        try {
            $filename = Amibios_Library::PATH_FIRMWARE . '/' . $firmware;

            $file = new File(CLEAROS_TEMP_DIR . '/' . Amibios_Library::FILE_FIRMWARE_INSTALL_LOG);

            if ($file->exists())
                $file->delete();

            $options['validate_exit_code'] = FALSE;
            $options['log'] = Amibios_Library::FILE_FIRMWARE_INSTALL_LOG;
            
            $shell = new Shell();
            $shell->execute(Amibios_Library::COMMAND_AFU, $filename . ' /p /b /n /k /sp', TRUE, $options);

            // create run lock
            Amibios_Library::set_run_lock();

        } catch (Exception $e) {
            $lock->unlock();
            throw new Engine_Exception(clearos_exception_message($e));
        }

        $lock->unlock();

        // delete existing info log file
        $file = new File(CLEAROS_TEMP_DIR . '/' . Amibios_Library::FILE_FIRMWARE_INFO_LOG);

        if ($file->exists())
            $file->delete();

        $data = [];
        // TODO: return some helpful information

        return $data;
    }

    ///////////////////////////////////////////////////////////////////////////////
    // V A L I D A T I O N   R O U T I N E S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Validation routine for firmware.
     *
     * @param string $firmware firmware ID
     *
     * @return string error message if firmware is invalid or not installed.
     */

    public function validate_firmware($firmware)
    {
        clearos_profile(__METHOD__, __LINE__);

        $firmware_list = $this->get_list();

        if (!array_key_exists($firmware, $firmware_list))
            return lang('amibios_firmware_invalid');
    }
}
